@@ -7,8 +7,8 @@
 
 #include "ixion/cell_listener_tracker.hpp"
 #include "ixion/formula_name_resolver.hpp"
-#include "ixion/interface/formula_model_access.hpp"
 #include "ixion/cell.hpp"
+#include "ixion/model_context.hpp"
 
 #include <mdds/rectangle_set.hpp>
 
@@ -21,6 +21,8 @@
 #include <iostream>
 #endif
 
+#include "grouped_ranges.hpp"
+
 using namespace std;
 
 namespace ixion {
@@ -33,11 +35,11 @@ typedef std::unordered_map<abs_range_t, cell_listener_tracker::address_set_type*
 
 class dirty_cell_inserter : public std::unary_function<cell_listener_tracker::address_set_type*, void>
 {
-    iface::formula_model_access& m_context;
+    model_context& m_context;
     dirty_formula_cells_t& m_dirty_cells;
     cell_listener_tracker::address_set_type& m_addrs;
 public:
-    dirty_cell_inserter(iface::formula_model_access& cxt, dirty_formula_cells_t& dirty_cells, cell_listener_tracker::address_set_type& addrs) :
+    dirty_cell_inserter(model_context& cxt, dirty_formula_cells_t& dirty_cells, cell_listener_tracker::address_set_type& addrs) :
         m_context(cxt), m_dirty_cells(dirty_cells), m_addrs(addrs) {}
 
     void operator() (const cell_listener_tracker::address_set_type* p)
@@ -61,15 +63,17 @@ public:
 
 struct cell_listener_tracker::impl
 {
-
-    iface::formula_model_access& m_context;
+    model_context& m_context;
 
     mutable range_query_set_type m_query_set; ///< used for fast lookup of range listeners.
     cell_store_type m_cell_listeners;         ///< store listeners for single cells.
     range_store_type m_range_listeners;       ///< store listeners for ranges.
     cell_listener_tracker::address_set_type m_volatile_cells;
 
-    impl(iface::formula_model_access& cxt) : m_context(cxt) {}
+    grouped_ranges m_grouped_ranges;
+
+    impl(model_context& cxt) :
+        m_context(cxt) {}
 
     ~impl()
     {
@@ -86,9 +90,6 @@ struct cell_listener_tracker::impl
 void cell_listener_tracker::impl::get_all_range_listeners_re(
     const abs_address_t& origin_target, const abs_address_t& target, dirty_formula_cells_t& listeners, address_set_type& listeners_addrs) const
 {
-#if DEBUG_CELL_LISTENER_TRACKER
-    __IXION_DEBUG_OUT__ << "--- begin: target address: " << m_context.get_name_resolver().get_name(target, false) << endl;
-#endif
     if (listeners_addrs.count(target))
     {
         // Target is included in the listener list.  No need to scan twice.
@@ -131,16 +132,9 @@ void cell_listener_tracker::impl::get_all_range_listeners_re(
     // Add new listeners to the caller's list.
     listeners.insert(new_listeners.begin(), new_listeners.end());
     listeners_addrs.insert(new_listeners_addrs.begin(), new_listeners_addrs.end());
-#if DEBUG_CELL_LISTENER_TRACKER
-    __IXION_DEBUG_OUT__ << "new listeners: ";
-    std::for_each(new_listeners_addrs.begin(), new_listeners_addrs.end(),
-                  cell_addr_printer(m_context.get_name_resolver()));
-    cout << endl;
-    __IXION_DEBUG_OUT__ << "--- end: target address: " << m_context.get_name_resolver().get_name(target, false) << endl;
-#endif
 }
 
-cell_listener_tracker::cell_listener_tracker(iface::formula_model_access& cxt) :
+cell_listener_tracker::cell_listener_tracker(model_context& cxt) :
     mp_impl(make_unique<impl>(cxt)) {}
 
 cell_listener_tracker::~cell_listener_tracker() {}
@@ -148,9 +142,12 @@ cell_listener_tracker::~cell_listener_tracker() {}
 void cell_listener_tracker::add(const abs_address_t& src, const abs_address_t& dest)
 {
 #if DEBUG_CELL_LISTENER_TRACKER
-    const formula_name_resolver& res = mp_impl->m_context.get_name_resolver();
-    __IXION_DEBUG_OUT__ << "adding - cell src: " << res.get_name(src, false)
-        << "  cell dest: " << res.get_name(dest, false) << endl;
+    {
+        abs_address_t origin(0,0,0);
+        auto res = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &mp_impl->m_context);
+        __IXION_DEBUG_OUT__ << "adding - cell src: " << res->get_name(src, origin, false)
+            << "  cell dest: " << res->get_name(dest, origin, false) << endl;
+    }
 #endif
     cell_store_type::iterator itr = mp_impl->m_cell_listeners.find(dest);
     if (itr == mp_impl->m_cell_listeners.end())
@@ -168,9 +165,12 @@ void cell_listener_tracker::add(const abs_address_t& src, const abs_address_t& d
 void cell_listener_tracker::add(const abs_address_t& cell, const abs_range_t& range)
 {
 #if DEBUG_CELL_LISTENER_TRACKER
-    const formula_name_resolver& res = mp_impl->m_context.get_name_resolver();
-    __IXION_DEBUG_OUT__ << "adding - cell: " << res.get_name(cell, false)
-        << "  range: " << res.get_name(range, false) << endl;
+    {
+        abs_address_t origin(0,0,0);
+        auto res = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &mp_impl->m_context);
+        __IXION_DEBUG_OUT__ << "adding - cell src: " << res->get_name(cell, origin, false)
+            << "  range dest: " << res->get_name(range, origin, false) << endl;
+    }
 #endif
     range_store_type::iterator itr = mp_impl->m_range_listeners.find(range);
     if (itr == mp_impl->m_range_listeners.end())
@@ -212,10 +212,6 @@ void cell_listener_tracker::remove(const abs_address_t& src, const abs_address_t
 
 void cell_listener_tracker::remove(const abs_address_t& cell, const abs_range_t& range)
 {
-#if DEBUG_CELL_LISTENER_TRACKER
-    const formula_name_resolver& res = mp_impl->m_context.get_name_resolver();
-    __IXION_DEBUG_OUT__ << "removing - cell: " << res.get_name(cell, false) << "  range: " << res.get_name(range, false) << endl;
-#endif
     range_store_type::iterator itr = mp_impl->m_range_listeners.find(range);
     if (itr == mp_impl->m_range_listeners.end())
         // No listeners for this range.  Bail out.
@@ -267,10 +263,6 @@ public:
 void cell_listener_tracker::get_all_cell_listeners(
     const abs_address_t& target, dirty_formula_cells_t& listeners) const
 {
-#if DEBUG_CELL_LISTENER_TRACKER
-    const formula_name_resolver& res = mp_impl->m_context.get_name_resolver();
-    __IXION_DEBUG_OUT__ << "target cell: " << res.get_name(target, false) << endl;
-#endif
     cell_store_type::const_iterator itr = mp_impl->m_cell_listeners.find(target);
     if (itr == mp_impl->m_cell_listeners.end())
         // This target cell has no listeners.
@@ -298,39 +290,27 @@ void cell_listener_tracker::get_all_cell_listeners(
 void cell_listener_tracker::get_all_range_listeners(
     const abs_address_t& target, dirty_formula_cells_t& listeners) const
 {
-#if DEBUG_CELL_LISTENER_TRACKER
-    __IXION_DEBUG_OUT__ << get_formula_result_output_separator() << endl;
-    __IXION_DEBUG_OUT__ << "get all range listeners for target " << mp_impl->m_context.get_name_resolver().get_name(target, false) << endl;
-#endif
-
     address_set_type listeners_addrs; // to keep track of circular references.
     mp_impl->get_all_range_listeners_re(target, target, listeners, listeners_addrs);
 }
 
-void cell_listener_tracker::print_cell_listeners(
-    const abs_address_t& target, const formula_name_resolver& resolver) const
+void cell_listener_tracker::add_grouped_range(
+    sheet_t sheet, const abs_rc_range_t& range, uintptr_t identity)
 {
-    {
-        address_t pos_display(target);
-        pos_display.set_absolute(false);
-        cout << "The following cells listen to cell "
-            << resolver.get_name(pos_display, abs_address_t(), false) << endl;
-    }
+    mp_impl->m_grouped_ranges.add(sheet, range, identity);
+}
 
-    cell_store_type::const_iterator itr = mp_impl->m_cell_listeners.find(target);
-    if (itr == mp_impl->m_cell_listeners.end())
-        // No one listens to this target.
-        return;
+void cell_listener_tracker::remove_grouped_range(sheet_t sheet, uintptr_t identity)
+{
+    return mp_impl->m_grouped_ranges.remove(sheet, identity);
+}
 
-    const address_set_type& addrs = *itr->second;
-    address_set_type::const_iterator itr2 = addrs.begin(), itr2_end = addrs.end();
-    for (; itr2 != itr2_end; ++itr2)
-    {
-        address_t pos_display(*itr2);
-        pos_display.set_absolute(false);
-        cout << "  cell " << resolver.get_name(pos_display, abs_address_t(), false) << endl;
-    }
+abs_rc_address_t cell_listener_tracker::move_to_grouped_range_origin(
+    sheet_t sheet, const abs_rc_address_t& pos) const
+{
+    return mp_impl->m_grouped_ranges.move_to_origin(sheet, pos);
 }
 
 }
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
