@@ -35,7 +35,7 @@ formula_tokens_t parse_formula_string(
     const formula_name_resolver& resolver, const char* p, size_t n)
 {
     lexer_tokens_t lxr_tokens;
-    formula_lexer lexer(p, n);
+    formula_lexer lexer(cxt.get_config(), p, n);
     lexer.tokenize();
     lexer.swap_tokens(lxr_tokens);
 
@@ -91,7 +91,7 @@ public:
                 m_os << token->get_value();
                 break;
             case fop_sep:
-                m_os << ",";
+                m_os << m_cxt.get_config().sep_function_arg;
                 break;
             case fop_function:
             {
@@ -193,15 +193,35 @@ void register_formula_cell(iface::formula_model_access& cxt, const abs_address_t
         // Not a formula cell. Bail out.
         return;
 
+    cell_listener_tracker& tracker = cxt.get_cell_listener_tracker();
+
     std::vector<const formula_token*> ref_tokens = cell->get_ref_tokens(cxt, pos);
-    std::for_each(ref_tokens.begin(), ref_tokens.end(),
-             formula_cell_listener_handler(cxt,
-                 pos, formula_cell_listener_handler::mode_add));
+
+    for (const formula_token* p : ref_tokens)
+    {
+        switch (p->get_opcode())
+        {
+            case fop_single_ref:
+            {
+                abs_address_t addr = p->get_single_ref().to_abs(pos);
+                tracker.add(pos, addr);
+                break;
+            }
+            case fop_range_ref:
+            {
+                abs_range_t range = p->get_range_ref().to_abs(pos);
+                tracker.add(pos, range);
+                break;
+            }
+            default:
+                ; // ignore the rest.
+        }
+    }
 
     // Check if the cell is volatile.
-    const formula_tokens_t* tokens = cxt.get_formula_tokens(pos.sheet, cell->get_identifier());
-    if (tokens && has_volatile(*tokens))
-        cxt.get_cell_listener_tracker().add_volatile(pos);
+    const formula_tokens_store_ptr_t& ts = cell->get_tokens();
+    if (ts && has_volatile(ts->get()))
+        tracker.add_volatile(pos);
 }
 
 void unregister_formula_cell(iface::formula_model_access& cxt, const abs_address_t& pos)
@@ -220,9 +240,28 @@ void unregister_formula_cell(iface::formula_model_access& cxt, const abs_address
     // itself as their listener.  This step is important
     // especially during partial re-calculation.
     std::vector<const formula_token*> ref_tokens = fcell->get_ref_tokens(cxt, pos);
-    for_each(ref_tokens.begin(), ref_tokens.end(),
-             formula_cell_listener_handler(cxt,
-                 pos, formula_cell_listener_handler::mode_remove));
+
+    for (const formula_token* p : ref_tokens)
+    {
+
+        switch (p->get_opcode())
+        {
+            case fop_single_ref:
+            {
+                abs_address_t addr = p->get_single_ref().to_abs(pos);
+                tracker.remove(pos, addr);
+                break;
+            }
+            case fop_range_ref:
+            {
+                abs_range_t range = p->get_range_ref().to_abs(pos);
+                tracker.remove(pos, range);
+                break;
+            }
+            default:
+                ; // ignore the rest.
+        }
+    }
 }
 
 void get_all_dirty_cells(
@@ -248,18 +287,26 @@ void get_all_dirty_cells(
         }
     }
 
+    // Get all range listeners first, then add the listeners to the list of
+    // modified cells, to get their listeners too.
+
+    dirty_formula_cells_t range_listeners;
+    for (const abs_address_t& addr : addrs)
+        tracker.get_all_range_listeners(addr, range_listeners);
+
+    for (const abs_address_t& cell : range_listeners)
+    {
+        addrs.push_back(cell);
+        cells.insert(cell);
+    }
+
     // Remove duplicate entries.
     std::sort(addrs.begin(), addrs.end());
     addrs.erase(std::unique(addrs.begin(), addrs.end()), addrs.end());
 
-    {
-        modified_cells_t::const_iterator itr = addrs.begin(), itr_end = addrs.end();
-        for (; itr != itr_end; ++itr)
-        {
-            tracker.get_all_range_listeners(*itr, cells);
-            tracker.get_all_cell_listeners(*itr, cells);
-        }
-    }
+    // Now get the single cell listeners.
+    for (const abs_address_t& addr : addrs)
+        tracker.get_all_cell_listeners(addr, cells);
 }
 
 void calculate_cells(iface::formula_model_access& cxt, dirty_formula_cells_t& cells, size_t thread_count)
