@@ -12,6 +12,7 @@
 #include "ixion/cell.hpp"
 #include "ixion/global.hpp"
 #include "ixion/formula_name_resolver.hpp"
+#include "ixion/matrix.hpp"
 #include "ixion/interface/formula_model_access.hpp"
 #include "ixion/interface/session_handler.hpp"
 #include "ixion/interface/table_handler.hpp"
@@ -106,7 +107,7 @@ bool formula_interpreter::interpret()
     catch (const formula_error& e)
     {
 #if DEBUG_FORMULA_INTERPRETER
-        __IXION_DEBUG_OUT__ << "formula error" << endl;
+        __IXION_DEBUG_OUT__ << "formula error: " << e.what() << endl;
 #endif
         if (mp_handler)
             mp_handler->set_formula_error(e.what());
@@ -120,9 +121,9 @@ bool formula_interpreter::interpret()
     return false;
 }
 
-const formula_result& formula_interpreter::get_result() const
+formula_result formula_interpreter::transfer_result()
 {
-    return m_result;
+    return std::move(m_result);
 }
 
 formula_error_t formula_interpreter::get_error() const
@@ -135,32 +136,28 @@ void formula_interpreter::init_tokens()
     name_set used_names;
     m_tokens.clear();
     m_stack.clear();
-    const formula_tokens_t* orig_tokens = NULL;
-    if (m_parent_cell->is_shared())
-        orig_tokens = m_context.get_shared_formula_tokens(m_pos.sheet, m_parent_cell->get_identifier());
-    else
-        orig_tokens = m_context.get_formula_tokens(m_pos.sheet, m_parent_cell->get_identifier());
 
-    if (!orig_tokens)
+    const formula_tokens_store_ptr_t& ts = m_parent_cell->get_tokens();
+    if (!ts)
         return;
 
-    formula_tokens_t::const_iterator itr = orig_tokens->begin(), itr_end = orig_tokens->end();
-    for (; itr != itr_end; ++itr)
+    const formula_tokens_t& src_tokens = ts->get();
+
+    for (const std::unique_ptr<formula_token>& p : src_tokens)
     {
-        const formula_token* p = &(**itr);
-        assert(p);
         if (p->get_opcode() == fop_named_expression)
         {
             // Named expression.  Expand it.
             const formula_tokens_t* expr = m_context.get_named_expression(
                 m_pos.sheet, p->get_name());
             used_names.insert(p->get_name());
-            expand_named_expression(p->get_name(), expr, used_names);
+            expand_named_expression(expr, used_names);
         }
         else
             // Normal token.
-            m_tokens.push_back(p);
+            m_tokens.push_back(p.get());
     }
+
     m_end_token_pos = m_tokens.end();
 }
 
@@ -201,29 +198,32 @@ void formula_interpreter::pop_result()
 {
     // there should only be one stack value left for the result value.
     assert(m_stack.size() == 1);
-    const stack_value& res = m_stack.back();
+    stack_value& res = m_stack.back();
     switch (res.get_type())
     {
         case stack_value_t::range_ref:
         {
             const abs_range_t& range = res.get_range();
             get_result_from_cell(m_context, range.first, m_result);
+            break;
         }
-        break;
         case stack_value_t::single_ref:
         {
             get_result_from_cell(m_context, res.get_address(), m_result);
+            break;
         }
-        break;
         case stack_value_t::string:
             m_result.set_string(res.get_string());
-        break;
+            break;
         case stack_value_t::value:
 #if DEBUG_FORMULA_INTERPRETER
             __IXION_DEBUG_OUT__ << "result: " << res.get_value() << endl;
 #endif
             m_result.set_value(res.get_value());
-        break;
+            break;
+        case stack_value_t::matrix:
+            m_result.set_matrix(res.pop_matrix());
+            break;
         default:
             ;
     }
@@ -232,8 +232,7 @@ void formula_interpreter::pop_result()
         mp_handler->set_result(m_result);
 }
 
-void formula_interpreter::expand_named_expression(
-    const string& expr_name, const formula_tokens_t* expr, name_set& used_names)
+void formula_interpreter::expand_named_expression(const formula_tokens_t* expr, name_set& used_names)
 {
     if (!expr)
         throw formula_error(formula_error_t::name_not_found);
@@ -251,9 +250,9 @@ void formula_interpreter::expand_named_expression(
                 // Circular reference detected.
                 throw invalid_expression("circular referencing of named expressions");
             }
-            const formula_tokens_t* expr = m_context.get_named_expression(m_pos.sheet, expr_name);
+            const formula_tokens_t* this_expr = m_context.get_named_expression(m_pos.sheet, expr_name);
             used_names.insert(expr_name);
-            expand_named_expression(expr_name, expr, used_names);
+            expand_named_expression(this_expr, used_names);
         }
         else
             m_tokens.push_back(&t);
@@ -356,7 +355,7 @@ bool pop_stack_value_or_string(const iface::formula_model_access& cxt,
                 {
                     const formula_cell* fc = cxt.get_formula_cell(addr);
                     assert(fc);
-                    const formula_result& res = fc->get_result_cache();
+                    formula_result res = fc->get_result_cache();
 
                     switch (res.get_type())
                     {
@@ -653,13 +652,12 @@ void formula_interpreter::factor()
             literal();
             return;
         default:
-        {
-            ostringstream os;
-            os << "factor: unexpected token type: <" << get_opcode_name(oc) << ">";
-            throw invalid_expression(os.str());
-        }
+            ;
     }
-    m_stack.push_value(0.0);
+
+    ostringstream os;
+    os << "factor: unexpected token type: <" << get_opcode_name(oc) << ">";
+    throw invalid_expression(os.str());
 }
 
 void formula_interpreter::paren()
