@@ -12,6 +12,8 @@
 #include "ixion/global.hpp"
 #include "ixion/macros.hpp"
 #include "ixion/interface/table_handler.hpp"
+#include "ixion/config.hpp"
+#include "ixion/matrix.hpp"
 
 #include <iostream>
 #include <cassert>
@@ -76,6 +78,65 @@ void test_string_pool()
     assert(s_val == cxt.get_string_identifier(IXION_ASCII("Value")));
 }
 
+void test_formula_tokens_store()
+{
+    formula_tokens_store_ptr_t p = formula_tokens_store::create();
+    assert(p->get_reference_count() == 1);
+    auto p2 = p;
+
+    assert(p->get_reference_count() == 2);
+    assert(p2->get_reference_count() == 2);
+
+    auto p3(p);
+
+    assert(p->get_reference_count() == 3);
+    assert(p2->get_reference_count() == 3);
+    assert(p3->get_reference_count() == 3);
+
+    p3.reset();
+    assert(p->get_reference_count() == 2);
+    assert(p2->get_reference_count() == 2);
+
+    p2.reset();
+    assert(p->get_reference_count() == 1);
+    p.reset();
+}
+
+void test_matrix()
+{
+    struct check
+    {
+        size_t row;
+        size_t col;
+        double val;
+    };
+
+    std::vector<check> checks =
+    {
+        { 0, 0, 1.0 },
+        { 0, 1, 2.0 },
+        { 1, 0, 3.0 },
+        { 1, 1, 4.0 },
+    };
+
+    numeric_matrix num_mtx(2, 2);
+
+    for (const check& c : checks)
+        num_mtx(c.row, c.col) = c.val;
+
+    for (const check& c : checks)
+        assert(num_mtx(c.row, c.col) == c.val);
+
+    matrix mtx(num_mtx);
+
+    for (const check& c : checks)
+    {
+        matrix::element e = mtx.get(c.row, c.col);
+        assert(e.type == matrix::element_type::numeric);
+        assert(e.numeric == c.val);
+    }
+};
+
 struct ref_name_entry
 {
     const char* name;
@@ -171,20 +232,24 @@ void test_name_resolver_excel_a1()
         assert(res.range.last.col == range_tests[i].col2);
     }
 
-    formula_name_t res = resolver->resolve("B1", 2, abs_address_t(0,1,1));
-    assert(res.type == formula_name_t::cell_reference);
-    assert(res.address.sheet == 0);
-    assert(res.address.row == -1);
-    assert(res.address.col == 0);
+    {
+        formula_name_t res = resolver->resolve("B1", 2, abs_address_t(0,1,1));
+        assert(res.type == formula_name_t::cell_reference);
+        assert(res.address.sheet == 0);
+        assert(res.address.row == -1);
+        assert(res.address.col == 0);
+    }
 
-    res = resolver->resolve("B2:B4", 5, abs_address_t(0,0,3));
-    assert(res.type == formula_name_t::range_reference);
-    assert(res.range.first.sheet == 0);
-    assert(res.range.first.row == 1);
-    assert(res.range.first.col == -2);
-    assert(res.range.last.sheet == 0);
-    assert(res.range.last.row == 3);
-    assert(res.range.last.col == -2);
+    {
+        formula_name_t res = resolver->resolve("B2:B4", 5, abs_address_t(0,0,3));
+        assert(res.type == formula_name_t::range_reference);
+        assert(res.range.first.sheet == 0);
+        assert(res.range.first.row == 1);
+        assert(res.range.first.col == -2);
+        assert(res.range.last.sheet == 0);
+        assert(res.range.last.row == 3);
+        assert(res.range.last.col == -2);
+    }
 
     // Parse name without row index.
     struct {
@@ -204,10 +269,11 @@ void test_name_resolver_excel_a1()
         assert(res.type == name_tests[i].type);
     }
 
-    // Parse address with non-existing sheet name.  It should be flagged invalid.
-
-    res = resolver->resolve(IXION_ASCII("NotExists!A1"), abs_address_t());
-    assert(res.type == formula_name_t::invalid);
+    {
+        // Parse address with non-existing sheet name.  It should be flagged invalid.
+        formula_name_t res = resolver->resolve(IXION_ASCII("NotExists!A1"), abs_address_t());
+        assert(res.type == formula_name_t::invalid);
+    }
 }
 
 void test_name_resolver_named_expression()
@@ -654,6 +720,7 @@ void test_parse_and_print_expressions()
     // Excel A1
 
     std::vector<const char*> exps = {
+        "\" \"",
         "1/3*1.4",
         "2.3*(1+2)/(34*(3-2))",
         "SUM(1,2,3)",
@@ -704,6 +771,27 @@ void test_parse_and_print_expressions()
         bool result = check_formula_expression(cxt, *resolver, exp);
         assert(result);
     }
+
+    // ODFF
+
+    exps = {
+        "\" \"",
+        "SUM([.A1];[.B1])",
+        "CONCATENATE([.A6];\" \";[.B6])",
+    };
+
+    resolver = formula_name_resolver::get(formula_name_resolver_t::odff, &cxt);
+    assert(resolver);
+
+    config cfg = cxt.get_config();
+    cfg.sep_function_arg = ';';
+    cxt.set_config(cfg);
+
+    for (const char* exp : exps)
+    {
+        bool result = check_formula_expression(cxt, *resolver, exp);
+        assert(result);
+    }
 }
 
 /**
@@ -747,7 +835,8 @@ formula_cell* insert_formula(
     model_context& cxt, const abs_address_t& pos, const char* exp,
     const formula_name_resolver& resolver)
 {
-    cxt.set_formula_cell(pos, exp, strlen(exp), resolver);
+    formula_tokens_t tokens = parse_formula_string(cxt, pos, resolver, exp, strlen(exp));
+    cxt.set_formula_cell(pos, std::move(tokens));
     register_formula_cell(cxt, pos);
     formula_cell* p = cxt.get_formula_cell(pos);
     assert(p);
@@ -781,7 +870,8 @@ void test_model_context_storage()
         // Test formula cells.
         abs_address_t pos(0,3,0);
         const char* exp = "SUM(1,2,3)";
-        cxt.set_formula_cell(pos, exp, strlen(exp), *resolver);
+        formula_tokens_t tokens = parse_formula_string(cxt, pos, *resolver, exp, strlen(exp));
+        cxt.set_formula_cell(pos, std::move(tokens));
         formula_cell* p = cxt.get_formula_cell(pos);
         assert(p);
     }
@@ -793,9 +883,9 @@ void test_model_context_storage()
 
         cxt.append_sheet(IXION_ASCII("test"), 1048576, 16384);
         string exp = "1";
-        cxt.set_formula_cell(abs_address_t(0,0,0), &exp[0], exp.size(), *resolver);
-        cxt.set_formula_cell(abs_address_t(0,2,0), &exp[0], exp.size(), *resolver);
-        cxt.set_formula_cell(abs_address_t(0,1,0), &exp[0], exp.size(), *resolver);
+        cxt.set_formula_cell(abs_address_t(0,0,0), parse_formula_string(cxt, abs_address_t(0,0,0), *resolver, &exp[0], exp.size()));
+        cxt.set_formula_cell(abs_address_t(0,2,0), parse_formula_string(cxt, abs_address_t(0,2,0), *resolver, &exp[0], exp.size()));
+        cxt.set_formula_cell(abs_address_t(0,1,0), parse_formula_string(cxt, abs_address_t(0,1,0), *resolver, &exp[0], exp.size()));
     }
 
     {
@@ -958,6 +1048,9 @@ int main()
     test_size();
     test_string_to_double();
     test_string_pool();
+    test_formula_tokens_store();
+    test_matrix();
+
     test_name_resolver_excel_a1();
     test_name_resolver_named_expression();
     test_name_resolver_table_excel_a1();
@@ -968,6 +1061,8 @@ int main()
     test_function_name_resolution();
     test_model_context_storage();
     test_volatile_function();
+
     return EXIT_SUCCESS;
 }
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
