@@ -17,12 +17,13 @@
 
 #include "workbook.hpp"
 #include "model_types.hpp"
+#include "calc_status.hpp"
 
-#include <memory>
 #include <sstream>
 #include <unordered_map>
 #include <map>
 #include <vector>
+#include <deque>
 #include <iostream>
 
 #define DEBUG_MODEL_CONTEXT 0
@@ -33,107 +34,6 @@ namespace ixion {
 
 namespace {
 
-struct delete_shared_tokens : public std::unary_function<model_context::shared_tokens, void>
-{
-    void operator() (const model_context::shared_tokens& v)
-    {
-        delete v.tokens;
-    }
-};
-
-class find_tokens_by_pointer : public std::unary_function<model_context::shared_tokens, bool>
-{
-    const formula_tokens_t* m_ptr;
-public:
-    find_tokens_by_pointer(const formula_tokens_t* p) : m_ptr(p) {}
-    bool operator() (const model_context::shared_tokens& v) const
-    {
-        return v.tokens == m_ptr;
-    }
-};
-
-/**
- * @return true if the formula cell is stored in the model with a shared
- *         formula token set, false if the formula cell has a non-shared
- *         formula token set, and is not yet stored in the model.
- */
-bool set_shared_formula_tokens_to_cell(
-    model_context& cxt, const abs_address_t& addr, formula_cell& fcell, const formula_tokens_t& new_tokens)
-{
-    if (addr.sheet == global_scope)
-        return false;
-
-    // Check its neighbors for adjacent formula cells.
-    if (addr.row == 0)
-        return false;
-
-    abs_address_t test = addr;
-    test.row -= 1;
-
-    if (cxt.get_celltype(test) != celltype_t::formula)
-        // The neighboring cell is not a formula cell.
-        return false;
-
-    formula_cell* test_cell = cxt.get_formula_cell(test);
-    if (!test_cell)
-        // The neighboring cell is not a formula cell.
-        throw general_error("formula cell doesn't exist but it should!");
-
-    if (test_cell->is_shared())
-    {
-        size_t token_id = test_cell->get_identifier();
-        const formula_tokens_t* tokens = cxt.get_shared_formula_tokens(addr.sheet, token_id);
-        assert(tokens);
-
-        if (new_tokens != *tokens)
-            return false;
-
-        // Make sure that we can extend the shared range properly.
-        abs_range_t range = cxt.get_shared_formula_range(addr.sheet, token_id);
-        if (range.first.sheet != addr.sheet)
-            // Wrong sheet
-            return false;
-
-        if (range.first.column != range.last.column)
-            // Must be a single column.
-            return false;
-
-        if (range.last.row != (addr.row - 1))
-            // Last row is not immediately above the current cell.
-            return false;
-
-        fcell.set_identifier(token_id);
-        fcell.set_shared(true);
-
-        range.last.row += 1;
-        cxt.set_shared_formula_range(addr.sheet, token_id, range);
-    }
-    else
-    {
-        size_t token_id = test_cell->get_identifier();
-        const formula_tokens_t* tokens = cxt.get_formula_tokens(addr.sheet, token_id);
-        assert(tokens);
-
-        if (new_tokens != *tokens)
-            return false;
-
-        // Move the tokens of the master cell to the shared token storage.
-        size_t shared_id = cxt.set_formula_tokens_shared(addr.sheet, token_id);
-        test_cell->set_shared(true);
-        test_cell->set_identifier(shared_id);
-        assert(test_cell->is_shared());
-        fcell.set_identifier(shared_id);
-        fcell.set_shared(true);
-        assert(fcell.is_shared());
-        abs_range_t range;
-        range.first = addr;
-        range.last = addr;
-        range.first.row -= 1;
-        cxt.set_shared_formula_range(addr.sheet, shared_id, range);
-    }
-    return true;
-}
-
 model_context::session_handler_factory dummy_session_handler_factory;
 
 } // anonymous namespace
@@ -143,10 +43,6 @@ class model_context_impl
     typedef std::vector<std::string> strings_type;
     typedef std::vector<std::unique_ptr<std::string>> string_pool_type;
     typedef std::unordered_map<mem_str_buf, string_id_t, mem_str_buf::hash> string_map_type;
-    typedef std::deque<formula_tokens_t*> formula_tokens_store_type;
-
-    typedef model_context::shared_tokens shared_tokens;
-    typedef model_context::shared_tokens_type shared_tokens_type;
 
 public:
     model_context_impl() = delete;
@@ -155,7 +51,6 @@ public:
 
     model_context_impl(model_context& parent) :
         m_parent(parent),
-        mp_config(new config),
         mp_cell_listener_tracker(new cell_listener_tracker(parent)),
         mp_table_handler(nullptr),
         mp_session_factory(&dummy_session_handler_factory)
@@ -164,16 +59,17 @@ public:
 
     ~model_context_impl()
     {
-        delete mp_config;
         delete mp_cell_listener_tracker;
-
-        for_each(m_tokens.begin(), m_tokens.end(), delete_element<formula_tokens_t>());
-        for_each(m_shared_tokens.begin(), m_shared_tokens.end(), delete_shared_tokens());
     }
 
     const config& get_config() const
     {
-        return *mp_config;
+        return m_config;
+    }
+
+    void set_config(const config& cfg)
+    {
+        m_config = cfg;
     }
 
     cell_listener_tracker& get_cell_listener_tracker()
@@ -211,8 +107,9 @@ public:
     void set_boolean_cell(const abs_address_t& addr, bool val);
     void set_string_cell(const abs_address_t& addr, const char* p, size_t n);
     void set_string_cell(const abs_address_t& addr, string_id_t identifier);
-    void set_formula_cell(const abs_address_t& addr, const char* p, size_t n, const formula_name_resolver& resolver);
-    void set_formula_cell(const abs_address_t& addr, size_t identifier, bool shared);
+    void set_formula_cell(const abs_address_t& addr, formula_tokens_t tokens);
+    void set_formula_cell(const abs_address_t& addr, const formula_tokens_store_ptr_t& tokens);
+    void set_grouped_formula_cells(const abs_range_t& group_range, formula_tokens_t tokens);
 
     abs_range_t get_data_range(sheet_t sheet) const;
 
@@ -235,7 +132,7 @@ public:
 
     sheet_t get_sheet_index(const char* p, size_t n) const;
     std::string get_sheet_name(sheet_t sheet) const;
-    sheet_size_t get_sheet_size(sheet_t sheet) const;
+    rc_size_t get_sheet_size(sheet_t sheet) const;
     size_t get_sheet_count() const;
     sheet_t append_sheet(const char* p, size_t n, row_t row_size, col_t col_size);
 
@@ -248,32 +145,6 @@ public:
     const column_store_t* get_column(sheet_t sheet, col_t col) const;
     const column_stores_t* get_columns(sheet_t sheet) const;
 
-    const formula_tokens_t* get_formula_tokens(sheet_t sheet, size_t identifier) const;
-    size_t add_formula_tokens(sheet_t sheet, formula_tokens_t* p);
-    void remove_formula_tokens(sheet_t sheet, size_t identifier);
-
-    void set_shared_formula(
-        const abs_address_t& addr, size_t si,
-        const char* p_formula, size_t n_formula,
-        const formula_name_resolver& resolver);
-
-    void set_shared_formula(
-        const abs_address_t& addr, size_t si,
-        const char* p_formula, size_t n_formula, const char* p_range, size_t n_range,
-        const formula_name_resolver& resolver);
-
-    void set_shared_formula(
-        const abs_address_t& addr, size_t si,
-        const char* p_formula, size_t n_formula, const abs_range_t& range,
-        const formula_name_resolver& resolver);
-
-    void set_shared_formula(const abs_address_t& addr, size_t si);
-
-    const formula_tokens_t* get_shared_formula_tokens(sheet_t sheet, size_t identifier) const;
-    size_t set_formula_tokens_shared(sheet_t sheet, size_t identifier);
-    abs_range_t get_shared_formula_range(sheet_t sheet, size_t identifier) const;
-    void set_shared_formula_range(sheet_t sheet, size_t identifier, const abs_range_t& range);
-
     double count_range(const abs_range_t& range, const values_t& values_type) const;
 
     dirty_formula_cells_t get_all_formula_cells() const;
@@ -287,15 +158,13 @@ private:
 
     workbook m_sheets;
 
-    config* mp_config;
+    config m_config;
     cell_listener_tracker* mp_cell_listener_tracker;
     iface::table_handler* mp_table_handler;
     detail::named_expressions_t m_named_expressions;
 
     model_context::session_handler_factory* mp_session_factory;
 
-    formula_tokens_store_type m_tokens;
-    model_context::shared_tokens_type m_shared_tokens;
     strings_type m_sheet_names; ///< index to sheet name map.
     string_pool_type m_strings;
     string_map_type m_string_map;
@@ -364,7 +233,7 @@ std::string model_context_impl::get_sheet_name(sheet_t sheet) const
     return m_sheet_names[sheet];
 }
 
-sheet_size_t model_context_impl::get_sheet_size(sheet_t sheet) const
+rc_size_t model_context_impl::get_sheet_size(sheet_t sheet) const
 {
     return m_sheets.at(sheet).get_sheet_size();
 }
@@ -481,153 +350,6 @@ const column_stores_t* model_context_impl::get_columns(sheet_t sheet) const
     return &sh.get_columns();
 }
 
-const formula_tokens_t* model_context_impl::get_formula_tokens(sheet_t sheet, size_t identifier) const
-{
-    if (m_tokens.size() <= identifier)
-        return NULL;
-    return m_tokens[identifier];
-}
-
-size_t model_context_impl::add_formula_tokens(sheet_t sheet, formula_tokens_t* p)
-{
-    // First, search for a NULL spot.
-    formula_tokens_store_type::iterator itr = std::find(
-        m_tokens.begin(), m_tokens.end(), static_cast<formula_tokens_t*>(NULL));
-
-    if (itr != m_tokens.end())
-    {
-        // NULL spot found.
-        size_t pos = std::distance(m_tokens.begin(), itr);
-        m_tokens[pos] = p;
-        return pos;
-    }
-
-    size_t identifier = m_tokens.size();
-    m_tokens.push_back(p);
-    return identifier;
-}
-
-void model_context_impl::remove_formula_tokens(sheet_t sheet, size_t identifier)
-{
-    if (m_tokens.size() >= identifier)
-        return;
-
-    delete m_tokens[identifier];
-    m_tokens[identifier] = NULL;
-}
-
-void model_context_impl::set_shared_formula(
-        const abs_address_t& addr, size_t si,
-        const char* p_formula, size_t n_formula, const char* p_range, size_t n_range,
-        const formula_name_resolver& resolver)
-{
-    formula_name_t name_type = resolver.resolve(p_range, n_range, abs_address_t());
-    abs_range_t range;
-    switch (name_type.type)
-    {
-        case ixion::formula_name_t::cell_reference:
-            range.first.sheet = name_type.address.sheet;
-            range.first.row = name_type.address.row;
-            range.first.column = name_type.address.col;
-            range.last = range.first;
-        break;
-        case ixion::formula_name_t::range_reference:
-            range.first.sheet = name_type.range.first.sheet;
-            range.first.row = name_type.range.first.row;
-            range.first.column = name_type.range.first.col;
-            range.last.sheet = name_type.range.last.sheet;
-            range.last.row = name_type.range.last.row;
-            range.last.column = name_type.range.last.col;
-        break;
-        default:
-        {
-            std::ostringstream os;
-            os << "failed to resolve shared formula range. ";
-            os << "(" << string(p_range, n_range) << ")";
-            throw general_error(os.str());
-        }
-    }
-
-    set_shared_formula(addr, si, p_formula, n_formula, range, resolver);
-}
-
-void model_context_impl::set_shared_formula(
-        const abs_address_t& addr, size_t si,
-        const char* p_formula, size_t n_formula, const abs_range_t& range,
-        const formula_name_resolver& resolver)
-{
-    // Tokenize the formula string and store it.
-    unique_ptr<formula_tokens_t> tokens = ixion::make_unique<formula_tokens_t>(
-        parse_formula_string(m_parent, addr, resolver, p_formula, n_formula));
-
-    if (si >= m_shared_tokens.size())
-        m_shared_tokens.resize(si+1);
-
-    m_shared_tokens[si].tokens = tokens.release();
-    m_shared_tokens[si].range = range;
-}
-
-void model_context_impl::set_shared_formula(
-        const abs_address_t& addr, size_t si,
-        const char* p_formula, size_t n_formula,
-        const formula_name_resolver& resolver)
-{
-    abs_range_t range;
-    range.first = addr;
-    range.last = range.first;
-    set_shared_formula(addr, si, p_formula, n_formula, range, resolver);
-}
-
-const formula_tokens_t* model_context_impl::get_shared_formula_tokens(sheet_t sheet, size_t identifier) const
-{
-#if DEBUG_MODEL_CONTEXT
-    __IXION_DEBUG_OUT__ << "identifier: " << identifier << "  shared token count: " << m_shared_tokens.size() << endl;
-#endif
-    if (m_shared_tokens.size() <= identifier)
-        return NULL;
-
-#if DEBUG_MODEL_CONTEXT
-    __IXION_DEBUG_OUT__ << "tokens: " << m_shared_tokens[identifier].tokens << endl;
-#endif
-    return m_shared_tokens[identifier].tokens;
-}
-
-size_t model_context_impl::set_formula_tokens_shared(sheet_t sheet, size_t identifier)
-{
-    assert(identifier < m_tokens.size());
-    formula_tokens_t* tokens = m_tokens.at(identifier);
-    assert(tokens);
-    m_tokens[identifier] = NULL;
-
-    // First, search for a NULL spot.
-    shared_tokens_type::iterator itr = std::find_if(
-        m_shared_tokens.begin(), m_shared_tokens.end(),
-        find_tokens_by_pointer(static_cast<const formula_tokens_t*>(NULL)));
-
-    if (itr != m_shared_tokens.end())
-    {
-        // NULL spot found.
-        size_t pos = std::distance(m_shared_tokens.begin(), itr);
-        m_shared_tokens[pos].tokens = tokens;
-        return pos;
-    }
-
-    size_t shared_identifier = m_shared_tokens.size();
-    m_shared_tokens.push_back(shared_tokens(tokens));
-    return shared_identifier;
-}
-
-abs_range_t model_context_impl::get_shared_formula_range(sheet_t sheet, size_t identifier) const
-{
-    assert(identifier < m_shared_tokens.size());
-    return m_shared_tokens.at(identifier).range;
-}
-
-void model_context_impl::set_shared_formula_range(sheet_t sheet, size_t identifier, const abs_range_t& range)
-{
-    m_shared_tokens.at(identifier).range = range;
-}
-
 namespace {
 
 double count_formula_block(
@@ -641,20 +363,24 @@ double count_formula_block(
     for (; pp != pp_end; ++pp)
     {
         const formula_cell& fc = **pp;
-        const formula_result& res = fc.get_result_cache();
+        formula_result res = fc.get_result_cache();
 
         switch (res.get_type())
         {
             case formula_result::result_type::value:
                 if (vt.is_numeric())
                     ++ret;
-            break;
+                break;
             case formula_result::result_type::string:
                 if (vt.is_string())
                     ++ret;
+                break;
             case formula_result::result_type::error:
                 // TODO : how do we handle error formula cells?
-            break;
+                break;
+            case formula_result::result_type::matrix:
+                // TODO : ditto
+                break;
         }
     }
 
@@ -790,16 +516,6 @@ void model_context_impl::erase_cell(const abs_address_t& addr)
     worksheet& sheet = m_sheets.at(addr.sheet);
     column_store_t& col_store = sheet.at(addr.column);
     column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
-
-    mdds::mtv::element_t celltype = col_store.get_type(addr.row);
-    if (celltype == element_type_formula)
-    {
-        const formula_cell* fcell = col_store.get<formula_cell*>(addr.row);
-        assert(fcell);
-        remove_formula_tokens(addr.sheet, fcell->get_identifier());
-    }
-
-    // Just update the hint. This call is not used during import.
     pos_hint = col_store.set_empty(addr.row, addr.row);
 }
 
@@ -825,7 +541,7 @@ void model_context_impl::set_string_cell(const abs_address_t& addr, const char*
     string_id_t str_id = add_string(p, n);
     column_store_t& col_store = sheet.at(addr.column);
     column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
-    pos_hint = col_store.set(pos_hint, addr.row, str_id);
+    pos_hint = col_store.set(pos_hint, addr.row, static_cast<unsigned long>(str_id));
 }
 
 void model_context_impl::set_string_cell(const abs_address_t& addr, string_id_t identifier)
@@ -833,39 +549,56 @@ void model_context_impl::set_string_cell(const abs_address_t& addr, string_id_t
     worksheet& sheet = m_sheets.at(addr.sheet);
     column_store_t& col_store = sheet.at(addr.column);
     column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
-    pos_hint = col_store.set(pos_hint, addr.row, identifier);
+    pos_hint = col_store.set(pos_hint, addr.row, static_cast<unsigned long>(identifier));
+}
+
+void model_context_impl::set_formula_cell(const abs_address_t& addr, formula_tokens_t tokens)
+{
+    formula_tokens_store_ptr_t ts = formula_tokens_store::create();
+    ts->get() = std::move(tokens);
+
+    set_formula_cell(addr, ts);
 }
 
 void model_context_impl::set_formula_cell(
-    const abs_address_t& addr, const char* p, size_t n, const formula_name_resolver& resolver)
+    const abs_address_t& addr, const formula_tokens_store_ptr_t& tokens)
 {
-    unique_ptr<formula_tokens_t> tokens =
-        ixion::make_unique<formula_tokens_t>(
-            parse_formula_string(m_parent, addr, resolver, p, n));
+    std::unique_ptr<formula_cell> fcell = ixion::make_unique<formula_cell>(tokens);
 
-    unique_ptr<formula_cell> fcell(new formula_cell);
-    if (!set_shared_formula_tokens_to_cell(m_parent, addr, *fcell, *tokens))
+    worksheet& sheet = m_sheets.at(addr.sheet);
+    column_store_t& col_store = sheet.at(addr.column);
+    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
+    pos_hint = col_store.set(pos_hint, addr.row, fcell.release());
+}
+
+void model_context_impl::set_grouped_formula_cells(
+    const abs_range_t& group_range, formula_tokens_t tokens)
+{
+    const abs_address_t& top_left = group_range.first;
+
+    rc_size_t group_size;
+    group_size.row    = group_range.last.row - group_range.first.row + 1;
+    group_size.column = group_range.last.column - group_range.first.column + 1;
+
+    formula_tokens_store_ptr_t ts = formula_tokens_store::create();
+    ts->get() = std::move(tokens);
+
+    calc_status_ptr_t cs(new calc_status(group_size));
+
+    worksheet& sheet = m_sheets.at(top_left.sheet);
+
+    for (col_t col_offset = 0; col_offset < group_size.column; ++col_offset)
     {
-        size_t tkid = add_formula_tokens(0, tokens.release());
-        fcell->set_identifier(tkid);
+        col_t col = top_left.column + col_offset;
+        column_store_t& col_store = sheet.at(col);
+        column_store_t::iterator& pos_hint = sheet.get_pos_hint(col);
+
+        for (row_t row_offset = 0; row_offset < group_size.row; ++row_offset)
+        {
+            row_t row = top_left.row + row_offset;
+            pos_hint = col_store.set(pos_hint, row, new formula_cell(row_offset, col_offset, cs, ts));
+        }
     }
-
-    worksheet& sheet = m_sheets.at(addr.sheet);
-    column_store_t& col_store = sheet.at(addr.column);
-    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
-    pos_hint = col_store.set(pos_hint, addr.row, fcell.release());
-}
-
-void model_context_impl::set_formula_cell(
-    const abs_address_t& addr, size_t identifier, bool shared)
-{
-    unique_ptr<formula_cell> fcell(new formula_cell(identifier));
-    fcell->set_shared(shared);
-
-    worksheet& sheet = m_sheets.at(addr.sheet);
-    column_store_t& col_store = sheet.at(addr.column);
-    column_store_t::iterator& pos_hint = sheet.get_pos_hint(addr.column);
-    pos_hint = col_store.set(pos_hint, addr.row, fcell.release());
 }
 
 abs_range_t model_context_impl::get_data_range(sheet_t sheet) const
@@ -1067,7 +800,7 @@ string_id_t model_context_impl::get_string_identifier(const abs_address_t& addr)
     switch (col_store.get_type(addr.row))
     {
         case ixion::element_type_string:
-            return col_store.get<string_id_t>(addr.row);
+            return col_store.get<unsigned long>(addr.row);
         default:
             ;
     }
@@ -1080,18 +813,20 @@ string_id_t model_context_impl::get_string_identifier_nowait(const abs_address_t
     switch (col_store.get_type(addr.row))
     {
         case ixion::element_type_string:
-            return col_store.get<string_id_t>(addr.row);
+            return col_store.get<unsigned long>(addr.row);
         case ixion::element_type_formula:
         {
             const formula_cell* p = col_store.get<formula_cell*>(addr.row);
-            const formula_result* res_cache = p->get_result_cache_nowait();
-            if (!res_cache)
+            formula_result res_cache = p->get_result_cache_nowait();
+            formula_result::result_type rt = res_cache.get_type();
+            if (rt == formula_result::result_type::error &&
+                res_cache.get_error() == formula_error_t::no_result_error)
                 break;
 
-            switch (res_cache->get_type())
+            switch (rt)
             {
                 case formula_result::result_type::string:
-                    return res_cache->get_string();
+                    return res_cache.get_string();
                 case formula_result::result_type::error:
                     // TODO : perhaps we should return the error string here.
                 default:
@@ -1134,21 +869,11 @@ std::unique_ptr<iface::session_handler> model_context::session_handler_factory::
     return std::unique_ptr<iface::session_handler>();
 }
 
-model_context::shared_tokens::shared_tokens() : tokens(NULL) {}
-model_context::shared_tokens::shared_tokens(formula_tokens_t* _tokens) : tokens(_tokens) {}
-model_context::shared_tokens::shared_tokens(const shared_tokens& r) : tokens(r.tokens), range(r.range) {}
-
-bool model_context::shared_tokens::operator== (const shared_tokens& r) const
-{
-    return tokens == r.tokens && range == r.range;
-}
-
 model_context::model_context() :
     mp_impl(new model_context_impl(*this)) {}
 
 model_context::~model_context()
 {
-    delete mp_impl;
 }
 
 const config& model_context::get_config() const
@@ -1186,16 +911,21 @@ void model_context::set_string_cell(const abs_address_t& addr, string_id_t ident
     mp_impl->set_string_cell(addr, identifier);
 }
 
-void model_context::set_formula_cell(
-    const abs_address_t& addr, const char* p, size_t n, const formula_name_resolver& resolver)
+void model_context::set_formula_cell(const abs_address_t& addr, formula_tokens_t tokens)
 {
-    mp_impl->set_formula_cell(addr, p, n, resolver);
+    mp_impl->set_formula_cell(addr, std::move(tokens));
 }
 
 void model_context::set_formula_cell(
-    const abs_address_t& addr, size_t identifier, bool shared)
+    const abs_address_t& addr, const formula_tokens_store_ptr_t& tokens)
 {
-    mp_impl->set_formula_cell(addr, identifier, shared);
+    mp_impl->set_formula_cell(addr, tokens);
+}
+
+void model_context::set_grouped_formula_cells(
+    const abs_range_t& group_range, formula_tokens_t tokens)
+{
+    mp_impl->set_grouped_formula_cells(group_range, std::move(tokens));
 }
 
 abs_range_t model_context::get_data_range(sheet_t sheet) const
@@ -1223,6 +953,11 @@ bool model_context::get_boolean_value(const abs_address_t& addr) const
     return mp_impl->get_boolean_value(addr);
 }
 
+void model_context::set_config(const config& cfg)
+{
+    mp_impl->set_config(cfg);
+}
+
 double model_context::get_numeric_value_nowait(const abs_address_t& addr) const
 {
     return mp_impl->get_numeric_value_nowait(addr);
@@ -1297,57 +1032,6 @@ const iface::table_handler* model_context::get_table_handler() const
     return mp_impl->get_table_handler();
 }
 
-const formula_tokens_t* model_context::get_formula_tokens(sheet_t sheet, size_t identifier) const
-{
-    return mp_impl->get_formula_tokens(sheet, identifier);
-}
-
-size_t model_context::add_formula_tokens(sheet_t sheet, formula_tokens_t* p)
-{
-    return mp_impl->add_formula_tokens(sheet, p);
-}
-
-void model_context::remove_formula_tokens(sheet_t sheet, size_t identifier)
-{
-    mp_impl->remove_formula_tokens(sheet, identifier);
-}
-
-void model_context::set_shared_formula(
-        const abs_address_t& addr, size_t si,
-        const char* p_formula, size_t n_formula, const char* p_range, size_t n_range,
-        const formula_name_resolver& resolver)
-{
-    mp_impl->set_shared_formula(addr, si, p_formula, n_formula, p_range, n_range, resolver);
-}
-
-void model_context::set_shared_formula(
-        const abs_address_t& addr, size_t si,
-        const char* p_formula, size_t n_formula,
-        const formula_name_resolver& resolver)
-{
-    mp_impl->set_shared_formula(addr, si, p_formula, n_formula, resolver);
-}
-
-const formula_tokens_t* model_context::get_shared_formula_tokens(sheet_t sheet, size_t identifier) const
-{
-    return mp_impl->get_shared_formula_tokens(sheet, identifier);
-}
-
-size_t model_context::set_formula_tokens_shared(sheet_t sheet, size_t identifier)
-{
-    return mp_impl->set_formula_tokens_shared(sheet, identifier);
-}
-
-abs_range_t model_context::get_shared_formula_range(sheet_t sheet, size_t identifier) const
-{
-    return mp_impl->get_shared_formula_range(sheet, identifier);
-}
-
-void model_context::set_shared_formula_range(sheet_t sheet, size_t identifier, const abs_range_t& range)
-{
-    return mp_impl->set_shared_formula_range(sheet, identifier, range);
-}
-
 string_id_t model_context::append_string(const char* p, size_t n)
 {
     return mp_impl->append_string(p, n);
@@ -1373,7 +1057,7 @@ std::string model_context::get_sheet_name(sheet_t sheet) const
     return mp_impl->get_sheet_name(sheet);
 }
 
-sheet_size_t model_context::get_sheet_size(sheet_t sheet) const
+rc_size_t model_context::get_sheet_size(sheet_t sheet) const
 {
     return mp_impl->get_sheet_size(sheet);
 }
@@ -1445,4 +1129,5 @@ bool model_context::empty() const
 }
 
 }
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
