@@ -11,6 +11,7 @@
 #include "ixion/formula_name_resolver.hpp"
 #include "ixion/formula_result.hpp"
 #include "ixion/macros.hpp"
+#include "ixion/address_iterator.hpp"
 
 #include <sstream>
 #include <iostream>
@@ -368,43 +369,68 @@ void model_parser::parse_init()
     if (cell_def.name.empty() && cell_def.value.empty())
         return;
 
-    m_dirty_cell_addrs.push_back(cell_def.pos);
-
-    switch (cell_def.type)
+    if (cell_def.matrix_value)
     {
-        case ct_formula:
-        {
-            m_context.set_formula_cell(cell_def.pos, cell_def.value.get(), cell_def.value.size(), *mp_name_resolver);
-            m_dirty_cells.insert(cell_def.pos);
+        assert(cell_def.type == ct_formula);
+        const abs_address_t& pos = cell_def.pos.first;
 
-            cout << get_display_cell_string(cell_def.pos) << ": (f) " << cell_def.value.str() << endl;
-            break;
-        }
-        case ct_string:
-        {
-            m_context.set_string_cell(cell_def.pos, cell_def.value.get(), cell_def.value.size());
+        formula_tokens_t tokens =
+            parse_formula_string(
+                m_context, pos, *mp_name_resolver, cell_def.value.get(), cell_def.value.size());
 
-            cout << get_display_cell_string(cell_def.pos) << ": (s) " << cell_def.value.str() << endl;
-            break;
-        }
-        case ct_value:
-        {
-            double v = global::to_double(cell_def.value.get(), cell_def.value.size());
-            m_context.set_numeric_cell(cell_def.pos, v);
+        m_context.set_grouped_formula_cells(cell_def.pos, std::move(tokens));
+        m_dirty_cells.insert(pos);
 
-            cout << get_display_cell_string(cell_def.pos) << ": (n) " << v << endl;
-            break;
-        }
-        case ct_boolean:
-        {
-            bool b = global::to_bool(cell_def.value.get(), cell_def.value.size());
-            m_context.set_boolean_cell(cell_def.pos, b);
+        cout << "{" << get_display_range_string(cell_def.pos) << "}: (m) " << cell_def.value.str() << endl;
+        return;
+    }
 
-            cout << get_display_cell_string(cell_def.pos) << ": (b) " << (b ? "true" : "false") << endl;
-            break;
+    abs_address_iterator iter(cell_def.pos, abs_address_iterator::direction_type::vertical);
+
+    for (const abs_address_t& pos : iter)
+    {
+        m_dirty_cell_addrs.push_back(pos);
+
+        switch (cell_def.type)
+        {
+            case ct_formula:
+            {
+                formula_tokens_t tokens =
+                    parse_formula_string(
+                        m_context, pos, *mp_name_resolver, cell_def.value.get(), cell_def.value.size());
+
+                m_context.set_formula_cell(pos, std::move(tokens));
+                m_dirty_cells.insert(pos);
+
+                cout << get_display_cell_string(pos) << ": (f) " << cell_def.value.str() << endl;
+                break;
+            }
+            case ct_string:
+            {
+                m_context.set_string_cell(pos, cell_def.value.get(), cell_def.value.size());
+
+                cout << get_display_cell_string(pos) << ": (s) " << cell_def.value.str() << endl;
+                break;
+            }
+            case ct_value:
+            {
+                double v = global::to_double(cell_def.value.get(), cell_def.value.size());
+                m_context.set_numeric_cell(pos, v);
+
+                cout << get_display_cell_string(pos) << ": (n) " << v << endl;
+                break;
+            }
+            case ct_boolean:
+            {
+                bool b = global::to_bool(cell_def.value.get(), cell_def.value.size());
+                m_context.set_boolean_cell(pos, b);
+
+                cout << get_display_cell_string(pos) << ": (b) " << (b ? "true" : "false") << endl;
+                break;
+            }
+            default:
+                throw model_parser::parse_error("unknown content type");
         }
-        default:
-            throw model_parser::parse_error("unknown content type");
     }
 }
 
@@ -414,59 +440,69 @@ void model_parser::parse_edit()
     if (cell_def.name.empty() && cell_def.value.empty())
         return;
 
-    m_dirty_cell_addrs.push_back(cell_def.pos);
-    unregister_formula_cell(m_context, cell_def.pos);
-
-    if (cell_def.value.empty())
+    if (cell_def.matrix_value)
     {
-        // A valid name is given but with empty definition.  Just remove the
-        // existing cell.
-        m_context.erase_cell(cell_def.pos);
+        assert(cell_def.type == ct_formula);
+        const abs_address_t& pos = cell_def.pos.first;
+
+        m_dirty_cell_addrs.push_back(pos);
+        unregister_formula_cell(m_context, pos);
+
+        formula_tokens_t tokens =
+            parse_formula_string(
+                m_context, pos, *mp_name_resolver, cell_def.value.get(), cell_def.value.size());
+
+        m_context.set_grouped_formula_cells(cell_def.pos, std::move(tokens));
+        m_dirty_cells.insert(pos);
+        register_formula_cell(m_context, pos);
         return;
     }
 
-    switch (cell_def.type)
-    {
-        case ct_formula:
-        {
-#if DEBUG_MODEL_PARSER
-            __IXION_DEBUG_OUT__ << "pos: " << resolver.get_name(cell_def.pos, false) << " type: formula" << endl;
-#endif
-            unregister_formula_cell(m_context, cell_def.pos);
-            m_context.set_formula_cell(cell_def.pos, cell_def.value.get(), cell_def.value.size(), *mp_name_resolver);
-            m_dirty_cells.insert(cell_def.pos);
-            register_formula_cell(m_context, cell_def.pos);
-            cout << cell_def.name.str() << ": (f) " << cell_def.value.str() << endl;
-#if DEBUG_MODEL_PARSER
-            std::string s = print_formula_tokens(m_context, cell_def.pos, *tokens);
-            __IXION_DEBUG_OUT__ << "formula tokens: " << s << endl;
-#endif
-        }
-        break;
-        case ct_string:
-        {
-#if DEBUG_MODEL_PARSER
-            __IXION_DEBUG_OUT__ << "pos: " << resolver.get_name(cell_def.pos, false) << " type: string" << endl;
-#endif
-            m_context.set_string_cell(cell_def.pos, cell_def.value.get(), cell_def.value.size());
-            cout << cell_def.name.str() << ": (s) " << cell_def.value.str() << endl;
-        }
-        break;
-        case ct_value:
-        {
-#if DEBUG_MODEL_PARSER
-            __IXION_DEBUG_OUT__ << "pos: " << resolver.get_name(cell_def.pos, false) << " type: numeric" << endl;
-#endif
-            double v = global::to_double(cell_def.value.get(), cell_def.value.size());
-            m_context.set_numeric_cell(cell_def.pos, v);
+    abs_address_iterator iter(cell_def.pos, abs_address_iterator::direction_type::vertical);
 
-            address_t pos_display(cell_def.pos);
-            pos_display.set_absolute(false);
-            cout << mp_name_resolver->get_name(pos_display, abs_address_t(), false) << ": (n) " << v << endl;
+    for (const abs_address_t& pos : iter)
+    {
+        m_dirty_cell_addrs.push_back(pos);
+        unregister_formula_cell(m_context, pos);
+
+        if (cell_def.value.empty())
+        {
+            // A valid name is given but with empty definition.  Just remove the
+            // existing cell.
+            m_context.erase_cell(pos);
+            continue;
+        }
+
+        switch (cell_def.type)
+        {
+            case ct_formula:
+            {
+                formula_tokens_t tokens =
+                    parse_formula_string(
+                        m_context, pos, *mp_name_resolver, cell_def.value.get(), cell_def.value.size());
+
+                m_context.set_formula_cell(pos, std::move(tokens));
+                m_dirty_cells.insert(pos);
+                register_formula_cell(m_context, pos);
+                cout << get_display_cell_string(pos) << ": (f) " << cell_def.value.str() << endl;
+            }
+            break;
+            case ct_string:
+            {
+                m_context.set_string_cell(pos, cell_def.value.get(), cell_def.value.size());
+                cout << get_display_cell_string(pos) << ": (s) " << cell_def.value.str() << endl;
+            }
+            break;
+            case ct_value:
+            {
+                double v = global::to_double(cell_def.value.get(), cell_def.value.size());
+                m_context.set_numeric_cell(pos, v);
+                cout << get_display_cell_string(pos) << ": (n) " << v << endl;
+            }
+            break;
+            default:
+                throw model_parser::parse_error("unknown content type");
         }
-        break;
-        default:
-            throw model_parser::parse_error("unknown content type");
     }
 }
 
@@ -721,43 +757,134 @@ model_parser::parsed_assignment_type model_parser::parse_assignment()
 
 model_parser::cell_def_type model_parser::parse_cell_definition()
 {
+    enum class section_type
+    {
+        name,
+        braced_name,
+        after_braced_name,
+        braced_value,
+        value
+    };
+
+    section_type section = section_type::name;
+
     cell_def_type ret;
     ret.type = model_parser::ct_unknown;
 
+    char skip_next = 0;
+
     mem_str_buf buf;
 
+    const char* line_head = mp_char;
+
     for (; mp_char != mp_end  && *mp_char != '\n'; ++mp_char)
     {
-        if (ret.name.empty() && is_separator(*mp_char))
+        if (skip_next)
         {
-            // Separator encountered.  Set the name and clear the buffer.
-            if (buf.empty())
-                throw model_parser::parse_error("left hand side is empty");
-
-            ret.name = buf;
-            buf.clear();
-            switch (*mp_char)
+            if (*mp_char != skip_next)
             {
-                case '=':
-                    ret.type = model_parser::ct_formula;
-                    break;
-                case ':':
-                    ret.type = model_parser::ct_value;
-                    break;
-                case '@':
-                    ret.type = model_parser::ct_string;
-                    break;
-                default:
-                    ;
+                std::ostringstream os;
+                os << "'" << skip_next << "' was expected, but '" << *mp_char << "' was found.";
+                throw model_parser::parse_error(os.str());
             }
+
+            skip_next = 0;
+            continue;
         }
-        else
+
+        switch (section)
         {
-            if (buf.empty())
-                buf.set_start(mp_char);
-            else
-                buf.inc();
+            case section_type::name:
+            {
+                if (mp_char == line_head && *mp_char == '{')
+                {
+                    section = section_type::braced_name;
+                    continue;
+                }
+
+                if (is_separator(*mp_char))
+                {
+                    // Separator encountered.  Set the name and clear the buffer.
+                    if (buf.empty())
+                        throw model_parser::parse_error("left hand side is empty");
+
+                    ret.name = buf;
+                    buf.clear();
+
+                    switch (*mp_char)
+                    {
+                        case '=':
+                            ret.type = model_parser::ct_formula;
+                            break;
+                        case ':':
+                            ret.type = model_parser::ct_value;
+                            break;
+                        case '@':
+                            ret.type = model_parser::ct_string;
+                            break;
+                        default:
+                            ;
+                    }
+
+                    section = section_type::value;
+                    continue;
+                }
+
+                break;
+            }
+            case section_type::braced_name:
+            {
+                if (*mp_char == '}')
+                {
+                    ret.name = buf;
+                    buf.clear();
+                    section = section_type::after_braced_name;
+                    continue;
+                }
+
+                break;
+            }
+            case section_type::after_braced_name:
+            {
+                switch (*mp_char)
+                {
+                    case '{':
+                        section = section_type::braced_value;
+                        ret.type = model_parser::ct_formula;
+                        skip_next = '=';
+                        break;
+                    case '=':
+                        ret.type = model_parser::ct_formula;
+                        section = section_type::value;
+                        break;
+                    case ':':
+                        ret.type = model_parser::ct_value;
+                        section = section_type::value;
+                        break;
+                    case '@':
+                        ret.type = model_parser::ct_string;
+                        section = section_type::value;
+                        break;
+                    default:
+                    {
+                        std::ostringstream os;
+                        os << "Unexpected character after braced name: '" << *mp_char << "'";
+                        throw model_parser::parse_error(os.str());
+                    }
+                }
+
+                continue; // skip this character.
+            }
+            case section_type::braced_value:
+            case section_type::value:
+            default:
+                ;
         }
+
+        if (buf.empty())
+            buf.set_start(mp_char);
+        else
+            buf.inc();
     }
 
     ret.value = buf;
@@ -769,8 +896,22 @@ model_parser::cell_def_type model_parser::parse_cell_definition()
             ret.type = model_parser::ct_boolean;
     }
 
+    if (section == section_type::braced_value)
+    {
+        // Make sure that the braced value ends with '}'.
+        char last = ret.value.back();
+        if (last != '}')
+        {
+            std::ostringstream os;
+            os << "'}' was expected at the end of a braced value, but '" << last << "' was found.";
+            model_parser::parse_error(os.str());
+        }
+        ret.value.dec();
+        ret.matrix_value = true;
+    }
+
 #if DEBUG_MODEL_PARSER
-    __IXION_DEBUG_OUT__ << "name: " << ret.name.str() << "  value: " << ret.value.str() << endl;
+    __IXION_DEBUG_OUT__ << "(name='" << ret.name.str() << "'; value='" << ret.value.str() << "')" << endl;
 #endif
 
     if (ret.name.empty())
@@ -788,15 +929,27 @@ model_parser::cell_def_type model_parser::parse_cell_definition()
     formula_name_t fnt = mp_name_resolver->resolve(
         ret.name.get(), ret.name.size(), abs_address_t(m_current_sheet,0,0));
 
-    if (fnt.type != formula_name_t::cell_reference)
+    switch (fnt.type)
     {
-        ostringstream os;
-        os << "invalid cell name: " << ret.name.str();
-        throw model_parser::parse_error(os.str());
+        case formula_name_t::cell_reference:
+        {
+            ret.pos.first = to_address(fnt.address).to_abs(abs_address_t(0,0,0));
+            ret.pos.last = ret.pos.first;
+            break;
+        }
+        case formula_name_t::range_reference:
+        {
+            ret.pos = to_range(fnt.range).to_abs(abs_address_t(0,0,0));
+            break;
+        }
+        default:
+        {
+            ostringstream os;
+            os << "invalid cell name: " << ret.name.str();
+            throw model_parser::parse_error(os.str());
+        }
     }
 
-    ret.pos = abs_address_t(fnt.address.sheet, fnt.address.row, fnt.address.col);
-
     return ret;
 }
 
@@ -832,7 +985,7 @@ void model_parser::check()
             case celltype_t::formula:
             {
                 const formula_cell* fcell = m_context.get_formula_cell(addr);
-                const formula_result& res_cell = fcell->get_result_cache();
+                formula_result res_cell = fcell->get_result_cache();
 
                 if (res_cell != res)
                 {
@@ -875,8 +1028,16 @@ void model_parser::check()
                     if (!ps)
                         throw check_error("failed to retrieve a string value for a string cell.");
 
+                    const string* ps_expected = m_context.get_string(res.get_string());
+                    if (!ps_expected)
+                    {
+                        std::ostringstream os;
+                        os << "no string value for the string ID of " << res.get_string();
+                        throw check_error(os.str());
+                    }
+
                     ostringstream os;
-                    os << "unexpected string result: (expected: " << res.get_string() << "; actual: " << *ps << ")";
+                    os << "unexpected string result: (expected: '" << *ps_expected << "'; actual: '" << *ps << "')";
                     throw check_error(os.str());
                 }
                 break;
@@ -894,6 +1055,14 @@ std::string model_parser::get_display_cell_string(const abs_address_t& pos) cons
     return mp_name_resolver->get_name(pos_display, abs_address_t(), m_print_sheet_name);
 }
 
+std::string model_parser::get_display_range_string(const abs_range_t& pos) const
+{
+    range_t pos_display(pos);
+    pos_display.first.set_absolute(false);
+    pos_display.last.set_absolute(false);
+    return mp_name_resolver->get_name(pos_display, abs_address_t(), m_print_sheet_name);
+}
+
 }
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
