@@ -63,7 +63,9 @@ class model_parser
         mem_str_buf name;
         mem_str_buf value;
         cell_type type;
-        abs_address_t pos;
+        abs_range_t pos;
+
+        bool matrix_value = false;
     };
 
 public:
@@ -117,6 +119,7 @@ private:
     void check();
 
     std::string get_display_cell_string(const abs_address_t& pos) const;
+    std::string get_display_range_string(const abs_range_t& pos) const;
 
 private:
 
