@@ -11,7 +11,7 @@ namespace ixion { namespace python {
 
 document_global::document_global() :
     m_cxt(),
-    m_resolver(ixion::formula_name_resolver::get(formula_name_resolver_excel_a1, &m_cxt))
+    m_resolver(ixion::formula_name_resolver::get(formula_name_resolver_t::excel_a1, &m_cxt))
 {}
 
 PyObject* get_python_document_error()
@@ -26,6 +26,12 @@ PyObject* get_python_sheet_error()
     return p;
 }
 
+PyObject* get_python_formula_error()
+{
+    static PyObject* p = PyErr_NewException(const_cast<char*>("ixion.FormulaError"), NULL, NULL);
+    return p;
+}
+
 }}
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
